@@ -1,10 +1,9 @@
 //! # Switch
 //!
-//! The switch is responsible for handling connections and messages as
-//! well as the cache on a dedicated thread. This module does not
-//! implement the actual listening logic, since that component is
-//! supposed to be more modularized. Instead it handles the thread and
-//! the cache, each protocol then has its own module.
+//! The Switch is responsible for forwarding messages from all
+//! components of the system to all others. It listens on multiple
+//! Channels and decides the targets based on type, origin, topic or
+//! target.
 
 use crate::error::Error;
 use crate::message::Message;
@@ -12,8 +11,7 @@ use crate::node::{Address, Center, Node};
 use crate::record::{Record, RecordBucket};
 use crate::router::Safe;
 use crate::signaling::{SignalingAction, Type};
-use crate::topic::Command;
-use crate::topic::TopicBucket;
+use crate::topic::{Command, TopicBucket};
 use crate::transaction::{Class, Transaction};
 use crate::util::Channel;
 use crate::InterfaceAction;
@@ -25,7 +23,14 @@ use std::time::SystemTime;
 /// server, which will autoamtically get started. The thread will hold
 /// a Switch object and send messages through the channel.
 pub struct Switch {
+    /// Channel to the Listener, sends and receives full Transactions,
+    /// which get serialized on demand when they are being sent.
     listener: Channel<Transaction>,
+    /// Channel for communicating with the User. The other end it held
+    /// by the Interface and it can be used for subscribing, sending
+    /// messages, sending actions. Should sending ever not be possible
+    /// this thread as well as all other threads of the system should
+    /// get shut down.
     interface: Channel<InterfaceAction>,
     signaling: Channel<SignalingAction>,
     /// The main copy of the couting table, which will be maintained
@@ -64,7 +69,7 @@ impl Switch {
             interface,
             signaling,
             table,
-            topics: RefCell::new(TopicBucket::new(center.clone())),
+            topics: RefCell::new(TopicBucket::new()),
             records,
             center,
         };
@@ -83,15 +88,18 @@ impl Switch {
             loop {
                 // 1. Listen on Interface Channel.
                 if let Some(action) = self.interface.try_recv() {
+                    log::info!("received action from the user");
                     match action {
                         InterfaceAction::Shutdown => {
-                            log::info!("received shutdown command, terminating Switch.");
+                            log::trace!("received shutdown request, terminating switch.");
                             break;
                         }
                         InterfaceAction::Message(transaction) => {
+                            log::trace!("received complete message from the user");
                             let _ = self.listener.send(transaction);
                         }
                         InterfaceAction::Subscribe(simple) => {
+                            log::trace!("received subscribe resquest from the user");
                             let target = simple.address.clone();
                             self.topics.borrow_mut().add(simple);
                             if self.table.should_be_local(&target) {
@@ -129,26 +137,51 @@ impl Switch {
                     }
                 }
 
+                let mut dropper: Address = Address::random();
+
                 // 2. Listen on topics Chanel.
-                for simple in &self.topics.borrow().topics {
+                for simple in self.topics.borrow().topics.iter() {
                     let topic = simple.address.clone();
                     if let Some(command) = simple.channel.try_recv() {
+                        log::info!("received message from topic");
                         match command {
                             Command::Drop(addr) => {
+                                log::trace!("topic went out of scope");
                                 // The addr is of the user to send the
                                 // unsubscribe to, not of the topic!
-                                self.topics.borrow_mut().remove(&simple.address);
-                                let message = Message::new(
-                                    Class::Unsubscribe,
-                                    self.center.public.clone(),
-                                    addr,
-                                    topic,
-                                    Vec::new(),
-                                );
-                                let t = Transaction::new(message);
-                                let _ = self.listener.send(t);
+                                dropper = simple.address.clone();
+                                let topic = simple.address.clone();
+                                if self.table.should_be_local(&topic) {
+                                    let message = Message::new(
+                                        Class::Unsubscribe,
+                                        self.center.public.clone(),
+                                        topic.clone(),
+                                        topic,
+                                        Vec::new(),
+                                    );
+                                    let t = Transaction::new(message);
+
+                                    Switch::handle_unsubscribe(
+                                        t,
+                                        &self.listener,
+                                        &self.records,
+                                        &self.topics,
+                                        &self.center,
+                                    )
+                                } else {
+                                    let message = Message::new(
+                                        Class::Unsubscribe,
+                                        self.center.public.clone(),
+                                        addr,
+                                        topic,
+                                        Vec::new(),
+                                    );
+                                    let t = Transaction::new(message);
+                                    let _ = self.listener.send(t);
+                                }
                             }
                             Command::Broadcast(addr, body) => {
+                                log::trace!("received broadcast from user");
                                 let message = Message::new(
                                     Class::Action,
                                     self.center.public.clone(),
@@ -164,10 +197,14 @@ impl Switch {
                     }
                 }
 
+                self.topics.borrow_mut().remove(&dropper);
+
                 // 3. Listen on Siganling Channel.
                 if let Some(action) = self.signaling.try_recv() {
+                    log::info!("received message from signaling thread");
                     match action.action {
                         Type::Ping => {
+                            log::trace!("received signaling ping request");
                             let message = Message::new(
                                 Class::Ping,
                                 self.center.public.clone(),
@@ -179,6 +216,7 @@ impl Switch {
                             let _ = self.listener.send(t);
                         }
                         Type::Lookup => {
+                            log::trace!("received signaling lookup request");
                             let message = Message::new(
                                 Class::Lookup,
                                 self.center.public.clone(),
@@ -195,30 +233,39 @@ impl Switch {
 
                 // 4. Listen on Handler Channel.
                 if let Some(t) = self.listener.try_recv() {
+                    log::info!("received message from listener");
                     let target = t.target();
                     if target == self.center.public {
+                        log::trace!("handling incoming message locally");
                         // Handle: Ping, Pong, Lookup, Details, Action, Subscriber, Unsubscriber
                         // Error: Subscriber, Unsubscribe
                         match t.class() {
                             Class::Ping => {
+                                log::trace!("incoming ping message");
                                 Switch::handle_ping(t, &self.listener, &self.center);
                             }
                             Class::Pong => {
+                                log::trace!("incoming pong message");
                                 Switch::handle_pong(t, &self.signaling);
                             }
                             Class::Lookup => {
+                                log::trace!("incoming lookup message");
                                 Switch::handle_lookup(t, &self.listener, &self.center);
                             }
                             Class::Details => {
+                                log::trace!("incoming details message");
                                 Switch::handle_details(t, &self.signaling, &self.table);
                             }
                             Class::Action => {
+                                log::trace!("incoming details message");
                                 Switch::handle_action(t, &self.topics, &self.interface);
                             }
                             Class::Subscriber => {
+                                log::trace!("incoming subscriber message");
                                 Switch::handle_subscriber(t, &self.topics, &self.center);
                             }
                             Class::Unsubscriber => {
+                                log::trace!("incoming unsubscriber message");
                                 Switch::handle_unsubscriber(t, &self.topics);
                             }
                             _ => {
@@ -226,10 +273,12 @@ impl Switch {
                             }
                         }
                     } else {
+                        log::trace!("target is not local but this node might be responsible");
                         // Forward: Ping, Pong, Details, Action, Subscriber, Unsubscriber,
                         // Maybe Handle: Subscribe, Unsubscribe, Lookup
                         match t.class() {
                             Class::Subscribe => {
+                                log::trace!("incoming subscribe message for local topic");
                                 Switch::handle_subscribe(
                                     t,
                                     &self.listener,
@@ -239,7 +288,14 @@ impl Switch {
                                 );
                             }
                             Class::Unsubscribe => {
-                                Switch::handle_unsubscribe(t, &self.listener, &self.records);
+                                log::trace!("incoming unsubscribe message for local topic");
+                                Switch::handle_unsubscribe(
+                                    t,
+                                    &self.listener,
+                                    &self.records,
+                                    &self.topics,
+                                    &self.center,
+                                );
                             }
                             // TODO: Handle lookup!
                             _ => {
@@ -373,22 +429,47 @@ impl Switch {
         }
     }
 
-    fn handle_unsubscribe(t: Transaction, listener: &Channel<Transaction>, records: &RecordBucket) {
-        let topic = t.topic();
+    fn handle_unsubscribe(
+        t: Transaction,
+        listener: &Channel<Transaction>,
+        records: &RecordBucket,
+        topics: &RefCell<TopicBucket>,
+        center: &Center,
+    ) {
+        let topic = t.target();
         match records.get(&topic) {
             Some(record) => {
+                let source = t.source();
                 records.unsubscribe(&topic, &t.source());
-
-                for sub in record.subscribers {
+                let mut subscribers = Vec::new();
+                record.subscribers.iter().for_each(|x| {
+                    if x != &source {
+                        subscribers.append(&mut x.as_bytes().to_vec())
+                    }
+                });
+                if topics.borrow().is_local(&topic) {
                     let message = Message::new(
-                        Class::Subscriber,
+                        Class::Unsubscriber,
+                        t.topic(),
                         t.source(),
-                        sub,
                         topic.clone(),
-                        Vec::new(),
+                        subscribers.clone(),
                     );
                     let transaction = Transaction::new(message);
-                    let _ = listener.send(transaction);
+                    Switch::handle_subscriber(transaction, topics, center)
+                }
+                for addr in record.subscribers {
+                    if addr != source {
+                        let message = Message::new(
+                            Class::Unsubscriber,
+                            t.source(),
+                            addr,
+                            topic.clone(),
+                            subscribers.clone(),
+                        );
+                        let transaction = Transaction::new(message);
+                        let _ = listener.send(transaction);
+                    }
                 }
             }
             None => {}
